@@ -23,9 +23,11 @@
 #include "lib/numpy.h"
 #include <Python.h>
 #include <numpy/ndarrayobject.h>
+#include <errno.h>
 
 namespace{
 
+
 const char* get_blob(PyObject* data, size_t& len) {
 #if PY_MAJOR_VERSION < 3
     if (!PyString_Check(data)) return 0;
@@ -49,6 +51,42 @@ const char* get_cstring(PyObject* stro) {
 #endif
 }
 
+
+options_map parse_options(PyObject* dict) {
+    options_map res;
+    if (!PyDict_Check(dict)) return res;
+    PyObject *key, *value;
+    Py_ssize_t pos = 0;
+
+    while (PyDict_Next(dict, &pos, &key, &value)) {
+        std::string k = get_cstring(key);
+        if (PyLong_Check(value)) {
+            res[k] = number_or_string(int(PyLong_AsLong(value)));
+#if PY_MAJOR_VERSION < 3
+        } else if (PyInt_Check(value)) {
+            res[k] = number_or_string(int(PyInt_AS_LONG(value)));
+#endif
+        } else if (PyFloat_Check(value)) {
+            res[k] = number_or_string(PyFloat_AS_DOUBLE(value));
+#if PY_MAJOR_VERSION >= 3
+        } else if (PyBytes_Check(value)) {
+            size_t len;
+            const char* blob = get_blob(value, len);
+            res[k] = number_or_string(std::string(blob, len));
+#endif
+        } else {
+            const char* c = get_cstring(value);
+            if (!c) {
+                std::stringstream ss;
+                ss << "Error while parsing option value for '" << k << "': type was not understood.";
+                throw OptionsError(ss.str());
+            }
+            res[k] = number_or_string(std::string(c));
+        }
+    }
+    return res;
+}
+
 const char TypeErrorMsg[] =
     "Type not understood. "
     "This is caused by either a direct call to _imread (which is dangerous: types are not checked!) or a bug in imread.py.\n";
@@ -61,6 +99,7 @@ PyObject* py_imread_may_multi(PyObject* self, PyObject* args, bool is_multi, boo
         PyErr_SetString(PyExc_RuntimeError,TypeErrorMsg);
         return NULL;
     }
+    options_map opts;
 
     try {
         std::auto_ptr<ImageFormat> format(get_format(formatstr));
@@ -99,14 +138,20 @@ PyObject* py_imread_may_multi(PyObject* self, PyObject* args, bool is_multi, boo
             int fd = ::open(filename, O_RDONLY|O_BINARY);
             if (fd < 0) {
                 std::stringstream ss;
-                ss << "File `" << filename << "` does not exist";
+                if (errno == EACCES) {
+                    ss << "Permission error when opening `" << filename << "`";
+                } else if (errno == ENOENT) {
+                    ss << "File `" << filename << "` does not exist";
+                } else {
+                    ss << "Unknown error opening `" << filename << "`.";
+                }
                 PyErr_SetString(PyExc_OSError, ss.str().c_str());
                 return 0;
             }
             input = std::auto_ptr<byte_source>(new fd_source_sink(fd));
         }
         if (is_multi) {
-            std::auto_ptr<image_list> images = format->read_multi(input.get(), &factory);
+            std::auto_ptr<image_list> images = format->read_multi(input.get(), &factory, opts);
             PyObject* output = PyList_New(images->size());
             if (!output) return NULL;
             std::vector<Image*> pages = images->release();
@@ -116,7 +161,7 @@ PyObject* py_imread_may_multi(PyObject* self, PyObject* args, bool is_multi, boo
             }
             return output;
         } else {
-            std::auto_ptr<Image> output = format->read(input.get(), &factory);
+            std::auto_ptr<Image> output = format->read(input.get(), &factory, opts);
             PyObject* final = PyTuple_New(2);
             if (!final) return NULL;
             PyTuple_SET_ITEM(final, 0, static_cast<NumpyImage&>(*output).releasePyObject());
@@ -139,48 +184,31 @@ PyObject* py_imread_from_blob   (PyObject* self, PyObject* args) { return py_imr
 PyObject* py_imsave(PyObject* self, PyObject* args) {
     const char* filename;
     const char* formatstr;
-    const char* meta = 0;
     PyObject* formatstrObject;
     PyArrayObject* array;
-    PyObject* metaObject;
     PyObject* asUtf8 = NULL;
-    if (!PyArg_ParseTuple(args, "sOOO", &filename, &formatstrObject, &array, &metaObject) || !PyArray_Check(array)) {
+    PyObject* optsDict;
+    if (!PyArg_ParseTuple(args, "sOOO", &filename, &formatstrObject, &array, &optsDict)) return NULL;
+    if (!PyArray_Check(array)) {
         PyErr_SetString(PyExc_RuntimeError,TypeErrorMsg);
         return NULL;
     }
 
-    // This is pretty ugly, mixing if and #if
-#if PY_MAJOR_VERSION < 3
-    if (PyString_Check(formatstrObject)) {
-        formatstr = PyString_AsString(formatstrObject);
-        if (metaObject != Py_None) {
-            if (!PyString_Check(metaObject)) {
-                PyErr_SetString(PyExc_RuntimeError,TypeErrorMsg);
-                return NULL;
-            }
-            meta = PyString_AsString(metaObject);
-        }
-#else
-    if (PyUnicode_Check(formatstrObject)) {
-        // On python 3.3, we can just do
-        //formatstr = PyUnicode_AsUTF8(formatstrObject);
-        asUtf8 = PyUnicode_AsUTF8String(formatstrObject);
-        if (!asUtf8) return NULL;
-        formatstr = PyBytes_AsString(asUtf8);
-        if (metaObject != Py_None) {
-            if (!PyBytes_Check(metaObject)) {
-                PyErr_SetString(PyExc_RuntimeError, "imread._imread: metadata is not Bytes");
-                return NULL;
-            }
-            meta = PyBytes_AsString(metaObject);
-        }
-#endif
-    } else {
+
+    formatstr = get_cstring(formatstrObject);
+    if (!formatstr) {
         PyErr_SetString(PyExc_TypeError, "imread.imsave: Expected a string as formatstr");
         return NULL;
     }
-    Py_INCREF(array);
     try {
+        options_map opts = parse_options(optsDict);
+        std::auto_ptr<ImageFormat> format(get_format(formatstr));
+        if (!format.get() || !format->can_write()) {
+            std::stringstream ss;
+            ss << "Cannot write this format (" << formatstr << ")";
+            throw CannotWriteError(ss.str());
+        }
+
         const int fd = ::open(filename, O_CREAT|O_WRONLY|O_BINARY|O_TRUNC, 0644);
         if (fd < 0) {
             std::stringstream ss;
@@ -188,16 +216,13 @@ PyObject* py_imsave(PyObject* self, PyObject* args) {
             throw CannotWriteError(ss.str());
         }
 
-        NumpyImage input(array);
         std::auto_ptr<byte_sink> output(new fd_source_sink(fd));
-        std::auto_ptr<ImageFormat> format(get_format(formatstr));
-        if (!format.get() || !format->can_write()) {
-            std::stringstream ss;
-            ss << "Cannot write this format (" << formatstr << ")";
-            throw CannotWriteError(ss.str());
-        }
-        if (meta && format->can_write_metadata()) format->write_with_metadata(&input, output.get(), meta);
-        else format->write(&input, output.get());
+
+        // ~NumpyImage() will decrease the count
+        Py_INCREF(array);
+        NumpyImage input(array);
+        format->write(&input, output.get(), opts);
+
         Py_XDECREF(asUtf8);
         Py_RETURN_NONE;
     } catch (const std::exception& e) {
