@@ -7,6 +7,8 @@
 #include <inttypes.h>
 #include <memory>
 #include <vector>
+#include <string>
+#include <map>
 #include <assert.h>
 
 #if defined(_MSC_VER)
@@ -142,6 +144,61 @@ struct image_list {
 };
 
 
+/// number_or_string is a sort of typed union.
+/// We could have used boost::any here, but that would have brought in a big
+/// dependency, which would otherwise not be used.
+struct number_or_string {
+    number_or_string()
+        :holds_(ns_empty)
+        { }
+
+    explicit number_or_string(std::string s)
+        :str_(s)
+        ,holds_(ns_string)
+        { }
+    explicit number_or_string(int i)
+        :int_(i)
+        ,holds_(ns_int)
+        { }
+    explicit number_or_string(double v)
+        :double_(v)
+        ,holds_(ns_double)
+        { }
+
+    bool get_int(int& n) const { if (holds_ != ns_int) return false; n = int_; return true; }
+    bool get_double(double& n) const { if (holds_ != ns_double) return false; n = double_; return true; }
+    bool get_str(std::string& s) const { if (holds_ != ns_string) return false; s = str_; return true; }
+    const char* maybe_c_str() const {
+        if (holds_ == ns_string) return str_.c_str();
+        return 0;
+    }
+
+    private:
+        std::string str_;
+        int int_;
+        double double_;
+        enum { ns_empty, ns_string, ns_int, ns_double } holds_;
+};
+
+
+typedef std::map<std::string, number_or_string> options_map;
+
+inline
+const char* get_optional_cstring(const options_map& opts, const std::string key) {
+    options_map::const_iterator iter = opts.find(key);
+    if (iter == opts.end()) return 0;
+    return iter->second.maybe_c_str();
+}
+
+inline
+int get_optional_int(const options_map& opts, const std::string key, const int def) {
+    options_map::const_iterator iter = opts.find(key);
+    if (iter == opts.end()) return def;
+    int v;
+    if (iter->second.get_int(v)) { return v; }
+    return def;
+}
+
 class ImageFormat {
     public:
         virtual ~ImageFormat() { }
@@ -151,18 +208,15 @@ class ImageFormat {
         virtual bool can_write() const { return false; }
         virtual bool can_write_metadata() const { return false; }
 
-        virtual std::auto_ptr<Image> read(byte_source* src, ImageFactory* factory) {
+        virtual std::auto_ptr<Image> read(byte_source* src, ImageFactory* factory, const options_map&) {
             throw NotImplementedError();
         }
-        virtual std::auto_ptr<image_list> read_multi(byte_source* src, ImageFactory* factory) {
+        virtual std::auto_ptr<image_list> read_multi(byte_source* src, ImageFactory* factory, const options_map&) {
             throw NotImplementedError();
         }
-        virtual void write(Image* input, byte_sink* output) {
+        virtual void write(Image* input, byte_sink* output, const options_map&) {
             throw NotImplementedError();
         }
-        virtual void write_with_metadata(Image* input, byte_sink* output, const char*) {
-            this->write(input, output);
-        }
 };
 
 #endif // LPC_IMREAD_H_INCLUDE_GUARD_WED_FEB__1_16_34_50_WET_2012
