@@ -1,4 +1,4 @@
-// Copyright 2012 Luis Pedro Coelho <luis@luispedro.org>
+// Copyright 2012-2013 Luis Pedro Coelho <luis@luispedro.org>
 // License: MIT (see COPYING.MIT file)
 
 #include "formats.h"
@@ -7,7 +7,10 @@
 #include "_lsm.h"
 #include "_png.h"
 #include "_tiff.h"
+
+#ifndef IMREAD_EXCLUDE_WEBP
 #include "_webp.h"
+#endif
 
 #include <cstring>
 
@@ -17,7 +20,13 @@ std::auto_ptr<ImageFormat> get_format(const char* format) {
     if (!strcmp(format, "jpeg") || !strcmp(format, "jpg")) return std::auto_ptr<ImageFormat>(new JPEGFormat);
     if (!strcmp(format, "lsm")) return std::auto_ptr<ImageFormat>(new LSMFormat);
     if (!strcmp(format, "tiff") || !strcmp(format, "tif")) return std::auto_ptr<ImageFormat>(new TIFFFormat);
+
+#if IMREAD_EXCLUDE_WEBP
+    if (!strcmp(format, "webp")) return std::auto_ptr<ImageFormat>(0);
+#else
     if (!strcmp(format, "webp")) return std::auto_ptr<ImageFormat>(new WebPFormat);
+#endif
+
     if (!strcmp(format, "stk")) return std::auto_ptr<ImageFormat>(new STKFormat);
     if (!strcmp(format, "bmp")) return std::auto_ptr<ImageFormat>(new BMPFormat);
     return std::auto_ptr<ImageFormat>(0);
