@@ -1,8 +1,7 @@
 from nose.tools import with_setup, raises
 import numpy as np
+from . import file_path
 from imread import imread, imsave
-from imread import _imread
-import numpy as np
 
 _filename = 'imread_testing_file.png'
 
@@ -17,7 +16,7 @@ def _remove_file():
 def test_png_raw():
     simple = np.arange(16*16).reshape((16,16))
     simple = simple.astype(np.uint8)
-    _imread.imsave(_filename, 'png', simple)
+    imsave(_filename, simple, 'png')
     back = imread(_filename)
     assert np.all(simple == back)
 
@@ -25,7 +24,7 @@ def test_png_raw():
 def test_asym():
     simple = np.arange(16*16).reshape((32,8))
     simple = simple.astype(np.uint8)
-    _imread.imsave(_filename, 'png', simple)
+    imsave(_filename, simple, 'png')
     back = imread(_filename)
     assert np.all(simple == back)
 
@@ -38,7 +37,7 @@ def test_random():
         simple = simple.astype(np.uint8)
         if i < 3:
             simple[:,:,i] = 0
-        _imread.imsave(_filename, 'png', simple)
+        imsave(_filename, simple, 'png')
         back = imread(_filename)
         assert np.all(simple == back)
 
@@ -67,3 +66,8 @@ def test_regression():
     im = imread('imread/tests/data/palette_zero.png')
     assert im.sum() == 0
     assert im.shape == (128, 64, 3)
+
+
+def test_16bit():
+    f = imread(file_path('arange512_16bit.png'))
+    assert np.all(f.ravel() == np.arange(512))
